import os
import logging
import re
import sys

logging.basicConfig()
logging.getLogger().setLevel(logging.INFO)


def is_paxlst(data):

    # mandatory EDIFACT segments
    edifact = ["UNB", "UNH", "UNT", "UNZ"]

    # mandatory PAXLST segments
    paxlst = ["ATT", "BGM", "CNT", "DOC", "NAD", "NAT", "TDT", "UNE", "UNG"]

    # for logging purposes only
    edifact_segments_missing = []
    for x in edifact:
        if x not in data:
            edifact_segments_missing.append(x)

    paxlst_segments_missing = []
    for x in paxlst:
        if x not in data:
            paxlst_segments_missing.append(x)

    # check if all mandatory EDIFACT segments are present
    if all(x in data for x in edifact):
        # check if all mandatory PAXLST segments are present
        if all(x in data for x in paxlst):

            # attempt to clean PAXLST file
            return True

        else:
            logging.error(
                'Not a valid PAXLST file. PAXLST segments missing: ' +
                str(paxlst_segments_missing))
            return False

    else:
        logging.error('Not a valid EDIFACT file. EDIFACT segments missing: ' +
                      str(edifact_segments_missing))
        return False


def clean(data):

    # trim new lines
    data = data.replace('\n', '').replace('\r', '')

    # trim hex characters
    data = re.sub(r'[^\x00-\x7f]', r'', data)

    if is_paxlst(data):

        # Service String Advice
        service_string_advice = "UNA"

        # Interchange Trailer; end of message segment
        interchange_trailer = "UNZ"

        # length of the SSA string according to PAXLST spec
        service_string_advice_length = 6

        # check if Service String Advice present
        if service_string_advice in data:

            # trim Default Service Characters
            default_service_characters = data[
                data.index(service_string_advice) +
                len(service_string_advice):data.index(service_string_advice) +
                len(service_string_advice) + service_string_advice_length]

            # Segment Terminator
            segment_terminator = default_service_characters[5]

            # beginning of PAXLST message
            # trim everythin before SSA string
            data = data[data.index(service_string_advice):]

            # end of PAXLST message
            # the PAXLST message ends after the Segment Terminator or the Interchange Trailer
            # i.e. UNZ...’ <-
            data = data[:data.index(segment_terminator,
                                    data.index(interchange_trailer)) + 1]

            # add newlines to increase readability
            paxlst = data.replace(segment_terminator,
                                  segment_terminator + "\n")

            # logging.info (paxlst)
            return (paxlst)

        # Service String Advice absent
        else:
            # assume default SSA: ":+.? '"
            segment_terminator = "'"

            # In case the Service String Advice("UNA") is missing,
            # the message starts with the Interchange Header("UNB")
            interchange_header = "UNB"

            data = data[data.index(interchange_header):]

            # end of PAXLST message
            # the PAXLST message ends after the Segment Terminator or the Interchange Trailer
            # i.e. UNZ...’ <-
            data = data[:data.index(segment_terminator,
                                    data.index(interchange_trailer)) + 1]

            # add newlines to increase readability
            paxlst = data.replace(segment_terminator,
                                  segment_terminator + "\n")

            # logging.info (paxlst)
            return (paxlst)


def cleanfile(filename):

    if os.path.isfile(filename):
        with open(filename, 'r') as file:

            filecontent = file.read()
            return clean(filecontent)

    else:
        logging.error("File does not exist: " + filename)


def cleandir(source_dir, target_dir, extension):

    if not os.path.isdir(source_dir):
        logging.error ("Source directory not found: " + source_dir)
        sys.exit()

    if not os.path.isdir(target_dir):
        logging.error ("Target directory not found: " + source_dir)
        sys.exit()

    for root, dirs, files in os.walk(source_dir):
        for file in sorted(files):
            target_content = cleanfile(os.path.join(root, file))

            if target_content:
                target = open(os.path.join(target_dir, file), "w")
                target.write(target_content)
                target.close()
                logging.info ("Cleaned " + os.path.join(target_dir, file))
            else:
                sys.exit()


from setuptools import setup, find_packages
import pathlib

here = pathlib.Path(__file__).parent.resolve()
long_description = (here / 'README.md').read_text(encoding='utf-8')

setup(
    name='ediclean',
    version='0.1.4',
    description=
    'A Python package to strip non-standard text blocks from UN/EDIFACT messages.',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://github.com/goOICT/ediclean',
    author='Zoltan Janota',
    author_email='zoltan.janota@un.org',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Topic :: Text Processing',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3',
    ],
    keywords='paxlst edifact un/edifact',
    packages=['ediclean'],
    python_requires='>=3.4',
    install_requires=[''],
    entry_points={
        'console_scripts': [
            'ediclean = ediclean.__main__:main',
        ],
    },
    project_urls={
        'Bug Reports': 'https://github.com/goOICT/ediclean/issues',
        'Source': 'https://github.com/goOICT/ediclean',
    },
)



import argparse
import os
import sys
import logging
import ediclean
import ediclean.paxlst as paxlst


def dir_path(path):
    if os.path.isdir(path):
        return path
    else:
        logging.error ("Directory does not exist: " + path)

def main() -> None:
    parser = argparse.ArgumentParser(
        prog='ediclean',
        description="Strip non-standard text blocks from UN/EDIFACT messages."
    )
    parser.add_argument('filename',
                        nargs='?',
                        help="File containing UN/EDIFACT PAXLST message")
    
    parser.add_argument('-s','--source_dir', type=dir_path)
    parser.add_argument('-t','--target_dir', type=dir_path)
    parser.add_argument('--version',
                    action="store_true",
                    help=argparse.SUPPRESS)

    try:
        args = parser.parse_args()
        if args.version:
            print("ediclean", ediclean.__version__)
        elif args.source_dir and not args.target_dir:
            print("Target directory missing")
        elif not args.source_dir and args.target_dir:
            print("Source directory missing")
        elif args.source_dir and args.target_dir:
            paxlst.cleandir(args.source_dir, args.target_dir, "")        
        elif not args.filename:
            parser.print_usage()
        elif args.filename:
            print(paxlst.cleanfile(args.filename))
        else:
            pass

    except SystemExit as err:
        if err.code == 2: parser.print_help()


if __name__ == '__main__':
    main()
